/*
  Button

  Turns on and off a light emitting diode(LED) connected to digital pin 13,
  when pressing a pushbutton attached to pin 2.

  The circuit:
  - LED attached from pin 13 to ground through 220 ohm resistor
  - pushbutton attached to pin 2 from +5V
  - 10K resistor attached to pin 2 from ground

  - Note: on most Arduinos there is already an LED on the board
    attached to pin 13.

  created 2005
  by DojoDave <http://www.0j0.org>
  modified 30 Aug 2011
  by Tom Igoe

  This example code is in the public domain.

  https://www.arduino.cc/en/Tutorial/BuiltInExamples/Button
*/
#include<LiquidCrystal.h>
#include "NNOCommandDefs.h"
#include "API.h"

// constants won't change. They're used here to set pin numbers:
const int onoff_btn = 2;     // the number of on/off button
const int scan_btn = 3;      // the number of scan button
const int up_btn = 4;        // the number of check last log
const int down_btn = 5;      // the number of check next log

#define en 1
#define rs 0
#define d7 9
#define d6 8
#define d5 7
#define d4 6

#define MAX_PACKET_SIZE 64
#define MIN_PACKET_SIZE 64
#define PASS 0

static unsigned char OutputBuffer[MAX_PACKET_SIZE+2];
static unsigned char InputBuffer[sizeof(nnoMessageStruct)+MIN_PACKET_SIZE];
static unsigned char seqNum=0;
static uint8 UARTWriteBuffer[1024 * 8];
static uint8 ReadBuffer[1024 * 8];
#define UARTMAXSIZE 8192
static bool g_UARTConnected = false;
int UART_Supported[62] = {1,1,1,1,1,1,1,1,1,1,0,1,1,1,0,0,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,
                          1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1};

int uartMesglength = 0;

LiquidCrystal lcd(rs,en,d4,d5,d6,d7);
// variables will change:
int onoffstate = 0;         // variable for reading the pushbutton status
int scanstate = 0;
int upstate = 0;
int downstate = 0;

int Serial_Read(unsigned char *Buffer,unsigned long Size){
//    while(Size){
//    Serial.println("Available bytes");
//    Serial.println(String(Serial.available()));
//    for (int i=12;i<Serial.available();i++){
//      ReadBuffer[i]=Serial.read();
//      Serial.print(ReadBuffer[i]);
//    }
//    
//    
////    InputBuffer=Serial.read();
////    Serial.println(receivedData);
////    NNO_Read();
//  }
//  while(Serial.available()){
//    Serial.println(Serial.read());
//  }
  int byteread=Serial.readBytes(Buffer,Size);
  Serial.println("byte read");
  Serial.println(byteread);
  return 1;
}

int Serial_Write(unsigned char *Buffer,unsigned long Size){
  int bytesent=Serial.write(Buffer,Size);
  Serial.println("byte sent");
  Serial.println(bytesent);
  return 1;
}

int NNO_SetUARTConnected(bool connected)
{
    g_UARTConnected = connected;
    return PASS;
}

static int NNO_PrepReadCmd(uint32 cmd_key)
/**
 * This function is private to this file. Prepares the read-control command packet for the given command code and copies it to OutputBuffer.
 *
 * @param   cmd  - I - USB command code.
 *
 * @return  0 = PASS
 *          -1 = FAIL
 *
 */
{
    nnoMessageStruct msg;

    msg.head.flags.rw = 1; //Read
    msg.head.flags.reply = 1; //Host wants a reply from device
    msg.head.flags.dest = 0; //Projector Control Endpoint
    msg.head.flags.reserved = 0;
    msg.head.flags.resp = 0;
    msg.head.seq = 0;

    msg.payload.cmd = CMD_GET_CMD(cmd_key);
    msg.head.length = 2;

   if(cmd_key == NNO_CMD_FLASH_GET_CHKSUM)
    {
        msg.payload.data[2] = 0x00;
        msg.head.length += 1;
    }

    OutputBuffer[0]=0; // First byte is the report number
    memcpy(&OutputBuffer[1], &msg, (sizeof(msg.head)+sizeof(msg.payload.cmd) + msg.head.length));


    return PASS;
}
static int NNO_Read(int isUART = 1 )
/**
 * This function is private to this file. This function is called to write the read control command and then read back a complete HID packet over USB
 * to InputBuffer.
 *
 * @return  number of bytes read
 *          -2 = nack from target
 *          -1 = error reading
 *
 */
{

    nnoMessageStruct *pMsg = (nnoMessageStruct *)InputBuffer;
    short msg_len=MIN_PACKET_SIZE;

    if(g_UARTConnected && isUART)//UART_Supported[pMsg->payload.cmd])
    {
        memcpy(&UARTWriteBuffer[8],&OutputBuffer[1],65);
        pMsg  = (nnoMessageStruct *)&OutputBuffer[1];
        unsigned int Checksum = 0;
        for(int i = 8; i < pMsg->head.length + 12  ; i++)
        {
            Checksum += UARTWriteBuffer[i];
        }


        UARTWriteBuffer[4] = Checksum;
        UARTWriteBuffer[5] = Checksum >> 8;
        UARTWriteBuffer[6] = Checksum >> 16;
        UARTWriteBuffer[7] = Checksum >> 24;

        //write the head and tail
        UARTWriteBuffer[0] = UART_START_IND_BYTE_0;
        UARTWriteBuffer[1] = UART_START_IND_BYTE_1;
        UARTWriteBuffer[2] = UART_START_IND_BYTE_2;
        UARTWriteBuffer[3] = UART_START_IND_BYTE_3;

        UARTWriteBuffer[pMsg->head.length + 12] = UART_END_IND_BYTE_0;
        UARTWriteBuffer[pMsg->head.length + 13] = UART_END_IND_BYTE_1;
        UARTWriteBuffer[pMsg->head.length + 14] = UART_END_IND_BYTE_2;
        UARTWriteBuffer[pMsg->head.length + 15] = UART_END_IND_BYTE_3;


        nnouartMessageStruct *uMsg = (nnouartMessageStruct *)ReadBuffer;
        int read_length = 0;
        int ret;
        if(Serial_Write(UARTWriteBuffer,pMsg->head.length + 16) == 1)
        {
            if(Serial_Read(ReadBuffer, 8 + sizeof(pMsg->head)) == 1)
            {
                read_length = uMsg->head.length;
            }

            else
                return FALSE;

            if((ret = Serial_Read(&ReadBuffer[12], read_length + 4)) == 1)
            {
                //skip the command name when copying from UART to USB structure
                memcpy(&InputBuffer[0],&ReadBuffer[8],4);
                memcpy(&InputBuffer[4],&ReadBuffer[14],read_length);
                return read_length;
            }
            else if(ret < 1)
            {
               return read_length;
            }
            else
                return FALSE;
        }
        return FALSE;
    }
    return FAIL;

}

static int NNO_GetAck(int isUART = 1)
/**
 * This function is private to this file. This function is called to get acknowledgement from device after a command is written
 *
 * @return  0 = ack from targe but no return value OR returns the 4 byte return value from target
 *          -2 = nack from target
 *          -1 = error reading
 *
 */
{
    nnoMessageStruct *pMsg = (nnoMessageStruct *)InputBuffer;
    if(g_UARTConnected && isUART)
    {

        int read_length = 0;
        nnouartMessageStruct *uMsg = (nnouartMessageStruct *)ReadBuffer;

        if(Serial_Read(ReadBuffer, 8 + sizeof(uMsg->head)) == 1)
        {
            read_length = uMsg->head.length;
        }

        else
            return FALSE;

        if(Serial_Read(&ReadBuffer[12], read_length + 4) == 1)
        {
            //skip the command name when copying from UART to USB structure
            memcpy(&InputBuffer[0],&ReadBuffer[8],4);
            memcpy(&InputBuffer[4],&ReadBuffer[14],read_length);

            if(pMsg->head.flags.resp == NNO_RESP_ERROR)
                return NNO_CMD_NACK;
            else if(pMsg->head.flags.resp == NNO_RESP_BUSY)
                return NNO_CMD_BUSY;
            else
            {
                if(pMsg->head.length != 4)
                    return PASS;
                else
                    return (pMsg->payload.data[0] | pMsg->payload.data[1] << 8 | pMsg->payload.data[2] << 16 | pMsg->payload.data[3] << 24);
            }
        }
        else
            return FALSE;

    }
//    else
//    {
//        int ret_val;
//
//
//        ret_val =  USB_Read(InputBuffer);
//
//        if(ret_val < 0)
//            return ret_val;
//        else if (ret_val == 0)
//            return NNO_READ_TIMEOUT;
//
//        if(pMsg->head.flags.resp == NNO_RESP_ERROR)
//            return NNO_CMD_NACK;
//        else if(pMsg->head.flags.resp == NNO_RESP_BUSY)
//            return NNO_CMD_BUSY;
//        else
//        {
//            if(pMsg->head.length != 4)
//                return PASS;
//            else
//                return (pMsg->payload.data[0] | pMsg->payload.data[1] << 8 | pMsg->payload.data[2] << 16 | pMsg->payload.data[3] << 24);
//        }
//    }
}

static int NNO_PrepWriteCmd(nnoMessageStruct *pMsg, uint32 cmd_key)
/**
 * This function is private to this file. Prepares the write command packet with given command code in the message structure pointer passed.
 *
 * @param   cmd  - I - USB command code.
 * @param   pMsg - I - Pointer to the message.
 *
 * @return  0 = PASS
 *          -1 = FAIL
 *
 */
{
    pMsg->head.flags.rw = 0; //Write
    //if(cmd == FILE_WRITE_DATA)
      // pMsg->head.flags.reply = 0; //No reply expected from device
   // else
        pMsg->head.flags.reply = 1; //Host wants a reply from device
    pMsg->head.flags.dest = 0; //Projector Control Endpoint
    pMsg->head.flags.reserved = 0;
    pMsg->head.flags.resp = 0;
    pMsg->head.seq = seqNum++;

    pMsg->payload.cmd = CMD_GET_CMD(cmd_key);
    pMsg->head.length = CMD_GET_LEN(cmd_key) + 2;



    return PASS;
}

static int NNO_SendMsg(nnoMessageStruct *pMsg, int isUART = 1)
/**
 * This function is private to this file. This function is called to send a whole HID message over USB; in chunks of 64 bytes.
 *
 * @return  number of bytes sent
 *          -1 = FAIL
 *
 */
{
    int maxDataSize = MAX_PACKET_SIZE-sizeof(pMsg->head);
    int dataBytesSent = MIN(pMsg->head.length, maxDataSize);    //Send all data or max possible
    Serial.println(g_UARTConnected);
    Serial.println(isUART);

    if(g_UARTConnected == true && isUART)
    {

        memcpy(&UARTWriteBuffer[8],pMsg,pMsg->head.length + 4);

        int length = pMsg->head.length + 16;
        unsigned int Checksum = 0;
        uartMesglength = length;
        for(int i = 8; i < pMsg->head.length + 12  ; i++)
        {
            Checksum += UARTWriteBuffer[i];
        }


        UARTWriteBuffer[4] = Checksum;
        UARTWriteBuffer[5] = Checksum >> 8;
        UARTWriteBuffer[6] = Checksum >> 16;
        UARTWriteBuffer[7] = Checksum >> 24;

        //write the head and tail
        UARTWriteBuffer[0] = UART_START_IND_BYTE_0;
        UARTWriteBuffer[1] = UART_START_IND_BYTE_1;
        UARTWriteBuffer[2] = UART_START_IND_BYTE_2;
        UARTWriteBuffer[3] = UART_START_IND_BYTE_3;

        UARTWriteBuffer[pMsg->head.length + 12] = UART_END_IND_BYTE_0;
        UARTWriteBuffer[pMsg->head.length + 13] = UART_END_IND_BYTE_1;
        UARTWriteBuffer[pMsg->head.length + 14] = UART_END_IND_BYTE_2;
        UARTWriteBuffer[pMsg->head.length + 15] = UART_END_IND_BYTE_3;

        Serial.println("in g_uartconnected if");
      
        int ret = Serial_Write(UARTWriteBuffer,length);
        Serial.println("print return ret");
        Serial.println(ret);
        return ret;

    }

//    OutputBuffer[0]=0; // First byte is the report number
//    memcpy(&OutputBuffer[1], pMsg, (sizeof(pMsg->head) + dataBytesSent));
//
////    if(NNO_Write() < 0)
////        return FAIL;
//
//    while(dataBytesSent < pMsg->head.length)
//    {
//        memcpy(&OutputBuffer[1], &pMsg->payload.data[dataBytesSent], MAX_PACKET_SIZE);
////        if(NNO_Write() < 0)
////            return FAIL;
//        dataBytesSent += MAX_PACKET_SIZE;
//    }
//    return dataBytesSent+sizeof(pMsg->head);
}

int NNO_DLPCEnable(bool enable,bool enable_lamp){
/**
 * Enables or Disables DLPC150; initializes DLPC150 for scans with patterns from Video interface or internal flash
 * and also enables/disables the lamp.
 *
 * (USB CMD OPCODE: BYTE1: 0x01, BYTE2: 0x05)
 *
 * @ingroup scanAPI
 *
 * @param enable - I - Releases DLPC150 from reset when TRUE and puts under
 * reset when FALSE.(as per scanType enum in API.h)
 * @param enable_lamp - I - true = turn on the lamp; false = leave lamp turned off.
 *
 * @return  PASS or FAIL
 *
 */
 {
    nnoMessageStruct msg;    int retval;

    msg.payload.data[2] = enable;
    msg.payload.data[3] = enable_lamp;
    int prepwritecmd=NNO_PrepWriteCmd(&msg, NNO_CMD_DLPC_ENABLE);
    Serial.println("prep write cmd");
    Serial.println(prepwritecmd);
    retval = NNO_SendMsg(&msg);
    Serial.println("retval ");
    Serial.println(retval);
    if(retval > 0)
    {
        int nnogetack_res=NNO_GetAck();
        Serial.println("NNO_Getack()");
        Serial.println(nnogetack_res);
        return nnogetack_res;
    }
    return FAIL;
}
  
}

int NNO_TestEEPROM(void)
/**
 * Sends command to perform EEPROM test.
 *
 * (USB CMD OPCODE: BYTE1: 0x01, BYTE2: 0x01)
 *
 * @ingroup testAPI
 *
 * @return  PASS or FAIL
 *
 */
{
    nnoMessageStruct msg;
    int ret_val;

    NNO_PrepReadCmd(NNO_CMD_EEPROM_TEST);
    if((ret_val = NNO_Read()) > 0)
    {
        memcpy(&msg, InputBuffer, 65);
        if(msg.payload.data[0] == 0)
            return PASS;
        else
            return FAIL;
    }
    Serial.println("return ret_val");
    return ret_val;
}

void setup() {
  // initialize the LED pin as an output:
//  pinMode(ledPin, OUTPUT);
  // initialize the pushbutton pin as an input:
//  pinMode(buttonPin, INPUT);
  pinMode(onoff_btn,INPUT);
  pinMode(scan_btn,INPUT);
  pinMode(up_btn,INPUT);
  pinMode(down_btn,INPUT);
  digitalWrite(onoff_btn,HIGH);
  digitalWrite(scan_btn,HIGH);
  digitalWrite(up_btn,HIGH);
  digitalWrite(down_btn,HIGH);
  Serial.begin(115200);

  lcd.begin(16,2);
  lcd.clear();
  lcd.print("Quick FN Start!");
}

void loop() {
  // read the state of the pushbutton value:

  onoffstate = digitalRead(onoff_btn);
  scanstate = digitalRead(scan_btn);
  upstate = digitalRead(up_btn);
  downstate = digitalRead(down_btn);
  
  
  // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
//  if (buttonState == HIGH) {
//    // turn LED on:
//    digitalWrite(ledPin, HIGH);
//  } else {
//    // turn LED off:
//    digitalWrite(ledPin, LOW);
//  }
  if (onoffstate == LOW){
    lcd.clear();
    lcd.setCursor(4,0);
    NNO_SetUARTConnected(!g_UARTConnected);
    Serial.println(g_UARTConnected);
    NNO_DLPCEnable(g_UARTConnected,g_UARTConnected);
//    if(g_UARTConnected){
//      Serial.println("DLP NIR On!");
//      lcd.print("DLP NIR On!");
//    }
//    else{
//      Serial.println("DLP NIR Off!");
//      lcd.print("DLP NIR Off!");
//    }
    Serial.println(NNO_TestEEPROM());
    
    
    
  }
  if (scanstate == LOW){
    Serial.println("Scan!");
    lcd.clear();
    lcd.setCursor(6,0);
    lcd.print("Scan!");
  }
  if (upstate == LOW){
    Serial.println("Last page!");
    lcd.clear();
    lcd.setCursor(4,0);
    lcd.print("Last page!");
  }
  if (downstate == LOW){
    Serial.println("Next Page!");
    lcd.clear();
    lcd.setCursor(4,0);
    lcd.print("Next page!");
  }

  
  delay(200);
}
